//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: 21 December 1997
// Last Modified: Sun Feb 15 18:44:35 GMT-0800 1998
// Filename:      ...sig/code/control/RadioDrum/RadioDrum.h
// Syntax:        C++
// $Smake:        cc -Wall -g -c %b.cpp -I../../../include && rm -f %b.o
//
// Description: A structure for handling the RadioDrum variables.
//


#ifndef _RADIODRUM_H_INCLUDED
#define _RADIODRUM_H_INCLUDED


#include "batonprotocol.h"
#include "MidiMessage.h"
#include "MidiIO.h"
#include "CircularBuffer.h"

#define DEFAULT_STATE_SIZE (100)
#define DATA_X             (0)
#define DATA_Y             (1)
#define DATA_Z             (2)
#define DATA_W             (2)


class RadioDrum : public MidiIO {
   public:
                  RadioDrum                (void);
                  RadioDrum                (int outputDevice, int inputDevice);
                 ~RadioDrum                ();

      int         getBuf                   (int index);
      void        processIncomingMessages  (void);
      int         getError                 (void) const;
      int         getPositionReporting     (void) const;
      int         getReportStatus          (void) const;
      long        getWhack1Time            (void) const;
      long        getWhack2Time            (void) const;
      void        requestBufData           (void);
      void        sendMessage              (int aMessage);
      void        setError                 (int errorState);
      int         recordingQ               (void) const;
      void        recordStateStart         (const char* aFilename);
      void        recordStateStop          (void);
      void        setReportStatus          (int aStatus);
      void        setStateSize             (int aSize);
      void        toggleError              (void);

      // messages to send out the the radio drum
      void        centerStick1             (void);
      void        centerStick2             (void);
      void        decreaseXYSensitivity    (void);
      void        decreaseZSensitivity     (void);
      void        increaseXYSensitivity    (void);
      void        increaseZSensitivity     (void);
      int         positionReportingOff     (void);
      int         positionReportingOn      (void);
      int         readCalibrationFile      (const char* aFilename);
      void        requestCalibration       (void);
      void        setStickLevels           (void);
      void        writeCalibrationFile     (const char* aFilename);
      int         testBaton                (void);
      void        togglePositionReporting  (void);
               

   // state variables
      CircularBuffer<uchar>  x1p;   // stick1 x-axis position
      CircularBuffer<uchar>  y1p;   // stick1 y-axis position
      CircularBuffer<uchar>  z1p;   // stick1 z-axis position
      CircularBuffer<uchar>  x2p;   // stick2 x-axis position
      CircularBuffer<uchar>  y2p;   // stick2 y-axis position
      CircularBuffer<uchar>  z2p;   // stick2 z-axis position   
      CircularBuffer<uchar>  d1p;   // dial 1 position
      CircularBuffer<uchar>  d2p;   // dial 2 position
      CircularBuffer<uchar>  d3p;   // dial 3 position
      CircularBuffer<uchar>  d4p;   // dial 4 position

      CircularBuffer<long>   t1t;   // trigger stick 1 time
      CircularBuffer<uchar>  x1t;   // stick1 x-axis trigger pos.
      CircularBuffer<uchar>  y1t;   // stick1 y-axis trigger pos.
      CircularBuffer<uchar>  w1t;   // stick1 wack at trigger time
      CircularBuffer<long>   t2t;   // trigger stick 2 time
      CircularBuffer<uchar>  x2t;   // stick2 x-axis trigger pos.
      CircularBuffer<uchar>  y2t;   // stick2 y-axis trigger pos.
      CircularBuffer<uchar>  w2t;   // stick2 whach at trigger time

      short calib[22];         // calibration data;
      char* calibFilename;     // the name of the file to write calib data
      int   writingCalib;      // set to true if writing calibration data
      short buf[16];           // antenna and potentiometer measurements
      int   completeBufQ[16];  // true if buffer index is not half complete

   protected:
      // temporary storage variables until complete data is received:
      uchar s1tf, x1t_temp, y1t_temp, w1t_temp; long t1t_temp;
      uchar s2tf, x2t_temp, y2t_temp, w2t_temp; long t2t_temp;
      uchar s1pf, x1p_temp, y1p_temp, z1p_temp; long t1p_temp;
      uchar s2pf, x2p_temp, y2p_temp, z2p_temp; long t2p_temp;

      // trigger times for buttons and pedals
      long b14pt, b15pt, b14mdt, b14mut, b15mdt, b15mut;

      int reportingQ;
      int errorQ;        // boolean to print error messages for unknown
                         // MIDI commands
   

      // recording variables 
      long        recordTimeOffset; 
      int         recordStateQ;
      fstream     recordOutput;
      void        recordState    (long aTime, const char* aState);
      void        recordState    (long aTime, const char* aState, int value);
      void        recordState    (long aTime, const char* aState,
                                    int value1, int value2, int value3);

      // private functions:
      void        interpretCommand      (MidiMessage aMessage);
      void        s1ts                  (long aTime);
      void        s1td                  (int flag, uchar aValue);
      void        s2ts                  (long aTime);
      void        s2td                  (int flag, uchar aValue);
      void        s1ps                  (long aTime);
      void        s1pd                  (int flag, uchar aValue);
      void        s2ps                  (long aTime);
      void        s2pd                  (int flag, uchar aValue);

      int         storedReportStatus;   // used for the next two functions
      void        pushReportingStatus   (void);
      void        popReportingStatus    (void);


      void        sendCalibration       (void);
      void        writeCalibrationData  (void);
      
};



#endif  // _RADIODRUM_H_INCLUDED



