//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sun Dec 28 15:18:46 GMT-0800 1997
// Last Modified: Mon Jan 12 15:42:44 GMT-0800 1998
// Filename:      ...sig/code/control/MidiOutPort/visual/MidiOutPort_visual.cpp
// Syntax:        C++ 
// $Smake:        cc -DVISUAL -O3 -o %b %b.cpp -lg++ -I../../../../include %
//                && strip %b
//


#ifdef VISUAL

#include <iostream.h>
#include "MidiOutPort_visual.h"

typedef unsigned long ulong;


// initialized static variables
int       MidiOutPort_visual::numDevices      = 0;
int       MidiOutPort_visual::objectCount     = 0;
int*      MidiOutPort_visual::openQ           = NULL;
int*      MidiOutPort_visual::portObjectCount = NULL;
HMIDIOUT* MidiOutPort_visual::device          = NULL;



//////////////////////////////
// 
// MidiOutPort_visual::MidiOutPort_visual
//	default values: autoOpen = 1
//


MidiOutPort_visual::MidiOutPort_visual(void) {
   if (objectCount == 0) {
      initialize();
   }
   objectCount++;

   port = -1;
   setPort(0);
}


MidiOutPort_visual::MidiOutPort_visual(int aPort, int autoOpen) {
   if (objectCount == 0) {
      initialize();
   }
   objectCount++;

   port = -1;
   setPort(aPort);
   if (autoOpen) {
      open();
   }
}



//////////////////////////////
//
// MidiOutPort_visual::~MidiOutPort_visual
//

MidiOutPort_visual::~MidiOutPort_visual() {
   objectCount--;
   if (objectCount == 0) {
      deinitialize();
   } else if (objectCount < 0) {
      cerr << "Error: bad MidiOutPort object count!: " << objectCount << endl;
      exit(1);
   }
}



//////////////////////////////
//
// MidiOutPort_visual::close
//

void MidiOutPort_visual::close(void) {
   if (getPortStatus() == 1 && device[getPort()] != NULL) {

      // The following function, midiOutClose, is not what I like.
      // It will send note offs to any note which it thinks is
      // on when the port is closed.  Uncomment the line if
      // you want this feature.
      // midiOutReset(device[getPort()]);

      midiOutClose(device[getPort()]);
      setPortStatus(0);
   }
}



//////////////////////////////
//
// MidiOutPort_visual::closeAll
//

void MidiOutPort_visual::closeAll(void) {
   for (int i=0; i<getNumPorts(); i++) {
      if (openQ[i] == 1 && device[i] != NULL) {
         midiOutReset(device[i]);
         midiOutClose(device[i]);
         openQ[i] = 0;
      }
   }
}



//////////////////////////////
//
// MidiOutPort_visual::getName -- returns the name of the port.
//	returns "" if no name. Name is valid until getName is called again.
//

const char* MidiOutPort_visual::getName(void) {
   static MIDIOUTCAPS outputCapabilities;
   if (openQ[getPort()]) {  // port already open
      midiOutGetDevCaps(getPort(), &outputCapabilities, sizeof(MIDIOUTCAPS));
   } else {  // port is currently closed
      if(open()) {;
         midiOutGetDevCaps(getPort(), &outputCapabilities, sizeof(MIDIOUTCAPS));
         close();
      } else {
         return "";
      }
   }
   return outputCapabilities.szPname;
}



//////////////////////////////
//
// MidiOutPort_visual::getNumPorts -- returns the number of available
// 	ports for MIDI output
//

int MidiOutPort_visual::getNumPorts(void) {
   return numDevices;
}



//////////////////////////////
//
// MidiOutPort_visual::getPort -- returns the port to which this
//	object belongs (as set with the setPort function).
//

int MidiOutPort_visual::getPort(void) {
   return port;
}



//////////////////////////////
//
// MidiOutPort_visual::getPortStatus -- 0 if closed, 1 if open
//

int MidiOutPort_visual::getPortStatus(void) {
   if (openQ[getPort()] == 1) {
      return 1;
   } else {
      return 0;
   }
}



//////////////////////////////
//
// MidiOutPort_visual::getTrace -- returns true if trace is on or
//	false if off.  If trace is on, then prints to standard output
//	the Midi message being sent.
//

int MidiOutPort_visual::getTrace(void) {
   return trace;
}



//////////////////////////////
//
// MidiOutPort_visual::rawsend -- send the Midi command and its parameters
//

int MidiOutPort_visual::rawsend(int command, int p1, int p2) {
   union { ulong word; uchar data[4]; } u;
   u.data[0] = (uchar)command;
   u.data[1] = (uchar)(p1 & 0x7f);  // parameter limited to range 0-127;
   u.data[2] = (uchar)(p2 & 0x7f);  // parameter limited to range 0-127;
   u.data[3] = 0;
  
   int flag = midiOutShortMsg(device[getPort()], u.word);
   
   if (getTrace()) {
      if (flag == MMSYSERR_NOERROR) {
         cout << "(" << hex << (int)u.data[0] << dec << ":"
              << (int)u.data[1] << "," << (int)u.data[2] << ")";
         cout.flush();
      } else {
         cout << "(" << hex << (int)u.data[0] << dec << "X"
              << (int)u.data[1] << "," << (int)u.data[2] << ")";
         cout.flush();
      }
   }

   return flag;
}


int MidiOutPort_visual::rawsend(int command, int p1) {
   return rawsend(command, p1, 0);
}


int MidiOutPort_visual::rawsend(int command) {
   return rawsend(command, 0, 0);
}


int MidiOutPort_visual::rawsend(uchar* array, int size) {
   cerr <<  "sysex data output not yet implemented." << endl;
   return 0;
}



//////////////////////////////
//
// MidiOutPort_visual::open -- returns true if MIDI output port was
//	opened.
//

int MidiOutPort_visual::open(void) {
   if (getPortStatus() == 0) {
      int flag;
      flag = midiOutOpen(&device[getPort()], getPort(), 0, 0, CALLBACK_NULL);
      if (flag == MMSYSERR_NOERROR) {
         openQ[getPort()] = 1;
         return 1;
      } else { // faied to open
         openQ[getPort()] = 0;
         device[getPort()] = NULL;
         return 0;
      }
   } else { // already open
      return 1;
   }
}



//////////////////////////////
//
// MidiOutPort_visual::setPort
//

void MidiOutPort_visual::setPort(int aPort) {
   if (aPort < 0 || aPort >= getNumPorts()) {
      cerr << "Error: maximum port number is: " << getNumPorts()-1
           << ", but you tried to access port: " << aPort << endl;
      exit(1);
   }

   if (port != -1) {
      portObjectCount[port]--;
   }
   port = aPort;
   portObjectCount[port]++;
}



//////////////////////////////
//
// MidiOutPort_visual::setTrace -- if false, then won't print
//      Midi messages to standard output.
//

int MidiOutPort_visual::setTrace(int aState) {
   int oldtrace = trace;
   if (aState == 0) {
      trace = 0;
   } else {
      trace = 1;
   }
   return oldtrace;
}



//////////////////////////////
//
// MidiOutPort_visual::toggleTrace
//

void MidiOutPort_visual::toggleTrace(void) {
   trace = !trace;
}



///////////////////////////////////////////////////////////////////////////
//
// Private functions
//



//////////////////////////////
//
// MidiOutPort_visual::deinitialize -- sets up storage if necessary
//	This function should be called if the current object is
//	the first object to be created.
//

void MidiOutPort_visual::deinitialize(void) {
   closeAll();
   if (device != NULL) delete [] device;
   device = NULL;
   if (openQ != NULL) delete [] openQ;
   openQ = NULL;
   if (portObjectCount != NULL) delete [] portObjectCount;
   portObjectCount = NULL;
}



//////////////////////////////
//
// MidiOutPort_visual::initialize -- sets up storage if necessary
//	This function should be called if the current object is
//	the first object to be created.
//

void MidiOutPort_visual::initialize(void) {
   // get the number of ports
   numDevices = midiOutGetNumDevs();
   if  (getNumPorts() <= 0) {
      cerr << "Error: no MIDI output devices" << endl;
      exit(1);
   }

   // allocate space for Windoze MIDI output structures
   if (device != NULL) {
      cerr << "Error: device array should be NULL when calling "
           << "initialize() in MidiOutPort." << endl;
      exit(1);
   }
   device = new HMIDIOUT[numDevices];

   // allocate space for openQ, the port open/close status
   if (openQ != NULL) delete [] openQ;
   openQ = new int[numDevices];

   // allocate space for object count on each port:
   if (portObjectCount != NULL) delete [] portObjectCount;
   portObjectCount = new int[numDevices];


   // initialize the static arrays
   for (int i=0; i<getNumPorts(); i++) {
      device[i] = NULL;
      openQ[i] = 0;
      portObjectCount[i] = 0;
   }
}



//////////////////////////////
//
// MidiOutPort_visual::setPortStatus
//

void MidiOutPort_visual::setPortStatus(int aStatus) {
   if (aStatus) {
      openQ[getPort()] = 1;
   } else {
      openQ[getPort()] = 0;
   }
}


#endif  // VISUAL




